/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlHistogramMemManager.cpp
** AUTHOR(S):   Olga Russakovsky <olga@cs.stanford.edu>
** DESCRIPTION:
**
*****************************************************************************/

#include "svlHistogramMemManager.h"

using namespace std;

svlHistogramMemManager::svlHistogramMemManager(unsigned num, bool allocateSpace) :
  _values(NULL),  _offset(0), _numValues(num), _ownsValues(allocateSpace)
{
  if (_ownsValues)
    _values = new vector<double>(_numValues, 0.0);
}

svlHistogramMemManager::svlHistogramMemManager(svlHistogramMemManager &m) :
  _offset(m._offset), _numValues(m._numValues), _ownsValues(m._ownsValues)
{
  if (_ownsValues)
    _values = new vector<double>(_numValues, 0.0);
  else
    _values = m._values;
}

svlHistogramMemManager::~svlHistogramMemManager()
{
  if (_ownsValues && _values)
    delete _values;
}

void svlHistogramMemManager::set(vector<double> *fv, unsigned offset)
{
  if (_ownsValues && _values)
    delete _values;

  _ownsValues = false;
  _values = fv;
  _offset = offset;
  if (_values->size() < _numValues + _offset)
    _values->resize(_numValues + _offset);
  clear();
}

void svlHistogramMemManager::clear()
{
  SVL_ASSERT_MSG(_values, "storage for the histogram must be provided");
  for (unsigned i = 0; i < _numValues; i++)
    _values->at(_offset + i) = 0.0;
}

void svlHistogramMemManager::normalize()
{
  SVL_ASSERT_MSG(_values, "storage for the histogram must be provided");
  double sum = 0.0;
  for (unsigned i = 0; i < _numValues; i++)
    sum += elem(i);

  if (abs(sum) < 1e-20) {
    SVL_LOG(SVL_LOG_WARNING, "histogram has sum close to 0, refusing to normalize");
    return;
  }

  for (unsigned i = 0; i < _numValues; i++)
    elem(i) /= sum;
}

void svlHistogramMemManager::truncate(double val)
{
  SVL_ASSERT_MSG(_values, "storage for the histogram must be provided");
  for (unsigned i = 0; i < _numValues; i++) {
    if (elem(i) > val)
      elem(i) = val;
  }
}

