% COMBINEIMAGES     Combines a number of images into a single image.
% Copyright (c) 2009, Stephen Gould <sgould@stanford.edu>
%

function [outImg] = combineImages(filenames, options);

% check input arguments
if (nargin < 1), help combineImages; outImg = []; return; end;
if (~iscell(filenames)), error('"filenames" must be a cell array'); end;

if (nargin < 2), options = []; end;
if (~isfield(options, 'width')), options.width = 64; end;
if (~isfield(options, 'height')), options.height = 48; end;
if (~isfield(options, 'spacing')), options.spacing = 1; end;

N = length(filenames);
Nr = ceil(sqrt(N));
Nc = ceil(N / Nr);

outImg = uint8(zeros(Nr * options.height + (Nr - 1) * options.spacing, ...
    Nc * options.width + (Nc - 1) * options.spacing, 3));

for i = 1:length(filenames),
    img = imread(filenames{i});
    % TODO: keep aspect ratio when resizing [H, W, C] = size(img);

    img = imresize(img, [options.height, options.width], 'bilinear');
    
    y = mod(i - 1, Nr) * (options.height + options.spacing);
    x = floor((i - 1) / Nr) * (options.width + options.spacing);
    
    outImg(y + (1:options.height), x + (1:options.width), :) = img;    
end;
